@@ -1,9 +1,16 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+from gettext import gettext as _
 from functools import wraps
+import json
 import bottle
+from bottle import request
 import plugin as lib_plugin
+from errors import ValidationError
+import importlib
+
+from . import jsonvalidation
 
 
 def route(path=None, method='GET', func=None, name=None, apply=None, skip=None, **config):
@@ -61,27 +68,42 @@ def delete(path=None, **kwargs):
     return _wrap(path=path, method='DELETE', **kwargs)
 
 
-def validate_schema(schema=None, **kwargs):
+def validate_schema(schema, **kwargs):
     """
-    Validate input according to some JSON-schema file, 
+    Validate input according to some JSON-schema file,
     and return an error object if there is a problem.
     """
     def decorator(callback):
-        print "Validating schema", schema
-        
-        template_dir = '.'
-        for root, dirs, files in os.walk(template_dir):
-            pass
-
-        validation_error = False
-
-        if validation_error:
-            callback = bottle.route(
-                path=rpath, method=method, callback=func, name=name,
-                apply=plugins, skip=skip, **config)(callback)
+        def convert_to_dict(bottle_form):
+            """
+            Function for converting a bottle multiple non-unique keys
+            dictionary to a normal python dictionary with lists automatically
+            created where appropriate, according to the JSON schema.
+            """
+            d = {}
+            for key, value in bottle_form.iterallitems():
+                if key not in d:
+                    d[key] = value
+                    if key in schema.get('properties') and not isinstance(value, list):
+                        if 'array' == schema['properties'][key].get('type') or 'array' in schema['properties'][key].get('type'):
+                            d[key] = [value]
+                else:
+                    if isinstance(d[key], list):
+                        d[key] += [value]
+                    else:
+                        d[key] = [d.get(key)] + [value]
+            return d
 
         @wraps(callback)
         def wrapper(*args, **kwargs):
+            instance = convert_to_dict(request.forms)
+            request.forms.as_dict = instance
+            errors = jsonvalidation.get_error_dictionary(
+                schema=schema, instance=instance)
+            if errors:
+                raise ValidationError(
+                    message=_("There were errors validating your request."),
+                    fields=errors)
             return callback(*args, **kwargs)
         return wrapper
     return decorator
\ No newline at end of file
