@@ -4,6 +4,8 @@
 import os
 import sys
 
+from errors import ValidationError
+
 
 class Dispatcher(object):
 
@@ -67,7 +69,8 @@ class Dispatcher(object):
 
     def is_controller(self, bundle, controller):
         path = os.path.join(self.config.apppath, 'bundles',
-                            bundle, 'controllers', '.'.join([controller, 'py']))
+                            bundle, 'controllers',
+                            '.'.join([controller, 'py']))
         return os.path.isfile(path)
 
     def __call__(self, environ, start_response):
